# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 CERN.
# Copyright (C) 2021-2022 Northwestern University.
# Copyright (C) 2021-2022 TU Wien.
#
# Invenio-Requests is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Results for the requests service."""

from invenio_records_resources.services.records.results import (
    FieldsResolver,
    RecordItem,
    RecordList,
)


class RequestItem(RecordItem):
    """Single request result."""

    def __init__(
        self,
        service,
        identity,
        request,
        errors=None,
        links_tpl=None,
        schema=None,
        expandable_fields=None,
        expand=False,
    ):
        """Constructor."""
        self._data = None
        self._errors = errors
        self._identity = identity
        self._request = request
        self._record = request
        self._service = service
        self._links_tpl = links_tpl
        self._schema = schema or service._wrap_schema(request.type.marshmallow_schema())
        self._fields_resolver = FieldsResolver(expandable_fields)
        self._expand = expand

    @property
    def id(self):
        """Identity of the request."""
        return str(self._request.id)

    def __getitem__(self, key):
        """Key a key from the data."""
        return self.data[key]

    @property
    def links(self):
        """Get links for this result item."""
        return self._links_tpl.expand(self._request, identity=self._identity)

    @property
    def _obj(self):
        """Return the object to dump."""
        return self._request

    @property
    def data(self):
        """Property to get the request."""
        if self._data:
            return self._data

        self._data = self._schema.dump(
            self._obj,
            context={
                "identity": self._identity,
                "record": self._request,
            },
        )

        if self._links_tpl:
            self._data["links"] = self.links

        if self._expand and self._fields_resolver:
            self._fields_resolver.resolve(self._identity, [self._data])
            fields = self._fields_resolver.expand(self._data)
            self._data["expanded"] = fields

        return self._data

    @property
    def errors(self):
        """Get the errors."""
        return self._errors

    def to_dict(self):
        """Get a dictionary for the request."""
        res = self.data
        if self._errors:
            res["errors"] = self._errors
        return res


class RequestList(RecordList):
    """List of request results."""

    def __init__(
        self,
        service,
        identity,
        results,
        params=None,
        links_tpl=None,
        links_item_tpl=None,
        expandable_fields=None,
        expand=False,
    ):
        """Constructor.

        :params service: a service instance
        :params identity: an identity that performed the service request
        :params results: the search results
        :params params: dictionary of the query parameters
        """
        self._identity = identity
        self._results = results
        self._service = service
        self._params = params
        self._links_tpl = links_tpl
        self._links_item_tpl = links_item_tpl
        self._fields_resolver = FieldsResolver(expandable_fields)
        self._expand = expand

    @property
    def hits(self):
        """Iterator over the hits."""
        request_cls = self._service.record_cls

        for hit in self._results:
            # load dump
            request = request_cls.loads(hit.to_dict())
            schema = self._service._wrap_schema(request.type.marshmallow_schema())

            # project the request
            projection = schema.dump(
                request,
                context={
                    "identity": self._identity,
                    "record": request,
                },
            )

            if self._links_item_tpl:
                projection["links"] = self._links_item_tpl.expand(
                    request, identity=self._identity
                )

            yield projection

    def to_dict(self):
        """Return result as a dictionary."""
        # TODO: This part should imitate the result item above. I.e. add a
        # "data" property which uses a ServiceSchema to dump the entire object.
        hits = list(self.hits)

        if self._expand and self._fields_resolver:
            self._fields_resolver.resolve(self._identity, hits)
            for hit in hits:
                fields = self._fields_resolver.expand(hit)
                hit["expanded"] = fields

        res = {
            "hits": {
                "hits": hits,
                "total": self.total,
            }
        }

        if self.aggregations:
            res["aggregations"] = self.aggregations

        if self._params:
            res["sortBy"] = self._params["sort"]
            if self._links_tpl:
                res["links"] = self._links_tpl.expand(self.pagination)

        return res



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Requests is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Request service results."""

from invenio_records_resources.services.records.results import ExpandableField

from ..resolvers.registry import ResolverRegistry


class EntityResolverExpandableField(ExpandableField):
    """Expandable entity resolver field.

    It will use the Entity resolver registry to retrieve the service to
    use to fetch records and the fields to return when serializing
    the referenced record.
    """

    entity_proxy = None

    def get_value_service(self, value):
        """Return the value and the service via entity resolvers."""
        self.entity_proxy = ResolverRegistry.resolve_entity_proxy(value)
        v = self.entity_proxy._parse_ref_dict_id()
        _resolver = self.entity_proxy.get_resolver()
        service = _resolver.get_service()
        return v, service

    def pick(self, resolved_rec):
        """Pick fields defined in the entity resolver."""
        return self.entity_proxy.pick_resolved_fields(resolved_rec)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Celery tasks for requests."""

from datetime import datetime

from celery import shared_task
from elasticsearch_dsl import Q
from elasticsearch_dsl.query import Bool
from flask import current_app
from invenio_access.permissions import system_identity

from .proxies import current_requests_service


@shared_task
def check_expired_requests():
    """Retrieve expired requests and perform expired action."""
    service = current_requests_service
    now = datetime.utcnow().isoformat()

    # using scan to get all requests
    requests_list = service.scan(
        identity=system_identity,
        extra_filter=Bool(
            "must",
            must=[
                # somehow querying for '"term", **{"is_expired: True"}' will not return any requests # noqa
                Q("range", **{"expires_at": {"lte": now}}),
                Q("term", **{"is_open": True}),
            ],
        ),
    )
    for r in requests_list:
        try:
            service.execute_action(
                identity=system_identity, id_=r["id"], action="expire"
            )
        except Exception as e:
            current_app.logger.warning(e)
            pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 TU Wien.
# Copyright (C) 2021 CERN.
#
# Invenio-Requests is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Type registry for looking up registered types per id.

The interface requires that the registered type have at least the property
``type_id`` defined:

.. code-block:: python

    class MyType(BaseType):
        type_id = "my-id"

Afterwards you can lookup types:

.. code-block:: python

    registry.lookup("my-id")
    for t in registry:
        # do something
"""


class TypeRegistry:
    """Registry for looking up registered types per id/name."""

    def __init__(self, types):
        """Constructor."""
        self._registered_types = {}
        for t in types:
            self.register_type(t)

    def register_type(self, type_, force=False):
        """Register the specified request_type."""
        type_id = type_.type_id

        if force:
            self._registered_types[type_id] = type_
        else:
            self._registered_types.setdefault(type_id, type_)

    def lookup(self, type_id, quiet=False, default=None):
        """Look up a registered type by its id."""
        if not quiet:
            return self._registered_types[type_id]

        return self._registered_types.get(type_id, default)

    def __iter__(self):
        """Iterate over all types."""
        for t in self._registered_types.values():
            yield t



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS bundles for communities.

You include one of the bundles in a page like the example below (using
``comunities`` bundle as an example):

.. code-block:: html

    {{ webpack['communities.js']}}

"""

from invenio_assets.webpack import WebpackThemeBundle

requests = WebpackThemeBundle(
    __name__,
    "assets",
    default="semantic-ui",
    themes={
        "semantic-ui": dict(
            entry={
                "invenio-requests-base": "./js/invenio_requests/requestsAppInit.js",
            },
            dependencies={
                "semantic-ui-css": "^2.4.1",
                "semantic-ui-react": "^2.1.1",
                "@ckeditor/ckeditor5-build-classic": "^16.0.0",
                "@ckeditor/ckeditor5-react": "^2.1.0",
                "axios": "^0.19.0",
                "lodash": "^4.17.15",
                "luxon": "^1.21.1",
                "path": "^0.12.7",
                "prop-types": "^15.7.2",
                "qs": "^6.9.1",
                "react": "^16.12.0",
                "react-dom": "^16.11.0",
                "yup": "^0.27.0",
                "react-overridable": "^0.0.3",
                "@semantic-ui-react/css-patch": "^1.0.0",
                "redux": "^4.0.5",
                "redux-devtools-extension": "^2.13.8",
                "redux-thunk": "^2.3.0",
                "react-redux": "^7.2.0",
                "i18next": "^20.3.0",
                "i18next-browser-languagedetector": "^6.1.0",
                "react-i18next": "^11.11.0",
            },
            aliases={
                "@translations/invenio_requests": "translations/invenio_requests",
                "@js/invenio_requests": "js/invenio_requests",
            },
        ),
    },
)
